@@ -26,11 +26,12 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
-#include <nautilus-burn-drive.h>
+#include <nautilus-burn.h>
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-url.h>
 #include <libgnomeui/gnome-ui-init.h>
@@ -90,8 +91,7 @@ static char *current_submit_url = NULL;
 
 gboolean autostart = FALSE, autoplay = FALSE;
 
-/* For the default drive stuff, TODO: remove */
-static GList *cdroms = NULL;
+static guint debug_flags;
 
 #define DEFAULT_PARANOIA 4
 #define RAISE_WINDOW "raise-window"
@@ -170,6 +170,42 @@ sj_main_set_title (const char* detail)
   }
 }
 
+void sj_debug (SjDebugDomain domain, const gchar* format, ...)
+{
+  va_list args;
+  gchar *string;
+
+  if (debug_flags & domain) {
+    va_start (args, format);
+    string = g_strdup_vprintf (format, args);
+    va_end (args);
+    g_printerr (string);
+    g_free (string);
+  }
+}
+
+static void sj_debug_init (void)
+{
+  const char *str;
+  const GDebugKey debug_keys[] = {
+    { "all", DEBUG_ALL },
+    { "cd", DEBUG_CD },
+    { "metadata", DEBUG_METADATA },
+    { "playing", DEBUG_PLAYING },
+    { "extracting", DEBUG_EXTRACTING }
+  };
+
+  str = g_getenv ("SJ_DEBUG");
+  if (str != NULL) {
+    debug_flags = g_parse_debug_string (str, debug_keys, G_N_ELEMENTS (debug_keys));
+  } else {
+    debug_flags = 0;
+  }
+  
+  if (debug_flags & DEBUG_ALL)
+    debug_flags = 0xFFFFFFFF;
+}
+
 static void error_on_start (GError *error)
 {
   GtkWidget *dialog;
@@ -352,6 +388,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_entry_set_text (GTK_ENTRY (artist_entry), album->artist);
     g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
+    /* Clear the genre field, it's from the user */
+    gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
 
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
@@ -421,7 +459,7 @@ AlbumDetails* multiple_album_dialog(GList *albums)
 
   if (dialog == NULL) {
     GtkTreeViewColumn *column;
-    GtkCellRenderer *text_renderer = text_renderer = gtk_cell_renderer_text_new ();
+    GtkCellRenderer *text_renderer = gtk_cell_renderer_text_new ();
 
     dialog = glade_xml_get_widget (glade, "multiple_dialog");
     g_assert (dialog != NULL);
@@ -561,6 +599,21 @@ void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gp
   }
 }
 
+/**
+ * The GConf key for audio volume changes
+ */
+void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+  g_assert (strcmp (entry->key, GCONF_AUDIO_VOLUME) == 0);
+  
+ GtkWidget *volb = glade_xml_get_widget (glade, "volume_button");
+ if (entry->value == NULL) {
+    bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), 1.0);
+  } else {
+  	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), gconf_value_get_float (entry->value));
+  }
+}
+
 static void
 metadata_cb (SjMetadata *m, GList *albums, GError *error)
 {
@@ -661,14 +714,15 @@ static void reread_cd (gboolean ignore_no_media)
   /* Set statusbar message */
   gtk_statusbar_push(GTK_STATUSBAR(status_bar), 0, _("Retrieving track listing...please wait."));
 
-  d(if (!drive) g_printerr("Attempting to re-read NULL drive\n"));
+  if (!drive)
+    sj_debug (DEBUG_CD, "Attempting to re-read NULL drive\n");
 
   g_free (current_submit_url);
   current_submit_url = NULL;
   gtk_widget_set_sensitive (submit_menuitem, FALSE);
 
   if (!is_audio_cd (drive)) {
-    d(g_printerr("Media is not an audio CD\n"));
+    sj_debug (DEBUG_CD, "Media is not an audio CD\n");
     update_ui_for_album (NULL);
     gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
     if (realized)
@@ -715,7 +769,7 @@ media_added_cb (NautilusBurnDrive *drive,
     /* FIXME: recover? */
   }
 
-  d(g_printerr ("** media added to device %s\n", drive->device));
+  sj_debug (DEBUG_CD, "Media added to device %s\n", nautilus_burn_drive_get_device (drive));
   reread_cd (TRUE);
 }
 
@@ -727,7 +781,7 @@ media_removed_cb (NautilusBurnDrive *drive,
     /* FIXME: recover? */
   }
 
-  d(g_printerr ("** media removed from device %s\n", drive->device));
+  sj_debug (DEBUG_CD, "Media removed from device %s\n", nautilus_burn_drive_get_device (drive));
   stop_ui_hack ();
   update_ui_for_album (NULL);
 }
@@ -736,7 +790,8 @@ static void
 set_drive_from_device (const char *device)
 {
   gboolean is_locked;
-  char    *reason;
+  char *reason;
+  NautilusBurnDriveMonitor *monitor;
 
   if (drive) {
     nautilus_burn_drive_unlock (drive);
@@ -746,8 +801,9 @@ set_drive_from_device (const char *device)
 
   if (! device)
     return;
-  
-  drive = nautilus_burn_drive_new_from_path (device);
+
+  monitor = nautilus_burn_get_drive_monitor ();
+  drive = nautilus_burn_drive_monitor_get_drive_for_device (monitor, device);
   g_assert (drive);
 
   is_locked = nautilus_burn_drive_lock (drive, _("Extracting audio from CD"), &reason);
@@ -756,7 +812,6 @@ set_drive_from_device (const char *device)
     g_free (reason);
   }
 
-  g_object_set (drive, "enable-monitor", TRUE, NULL);
   g_signal_connect (drive, "media-added", G_CALLBACK (media_added_cb), NULL);
   g_signal_connect (drive, "media-removed", G_CALLBACK (media_removed_cb), NULL);
 }
@@ -805,27 +860,41 @@ set_device (const char* device, gboolean ignore_no_media)
 
 gboolean cd_drive_exists (const char *device)
 {
-  GList *l;
-  if (device == NULL) return FALSE;
-  if (cdroms == NULL) {
-    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
-  }
-  for (l = cdroms; l != NULL; l = l->next) {
-    if (strcmp (((NautilusBurnDrive *) (l->data))->device, device) == 0)
-      return TRUE;
-  }
-  return FALSE;
+  NautilusBurnDriveMonitor *monitor;
+  NautilusBurnDrive *drive;
+  gboolean exists;
+
+  if (device == NULL)
+    return FALSE;
+
+  monitor = nautilus_burn_get_drive_monitor ();
+  drive = nautilus_burn_drive_monitor_get_drive_for_device (monitor, device);
+  exists = (drive != NULL);
+  nautilus_burn_drive_unref (drive);
+
+  return exists;
 }
 
-const char* prefs_get_default_device ()
+const char *
+prefs_get_default_device (void)
 {
-  static const char* default_device = NULL;
+  static const char * default_device = NULL;
+
   if (default_device == NULL) {
-    NautilusBurnDrive *cd;
-    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
-    if (cdroms == NULL) return NULL;
-    cd = cdroms->data;
-    default_device = cd->device;
+    NautilusBurnDriveMonitor *monitor;
+    NautilusBurnDrive *drive;
+    GList *drives;
+
+    monitor = nautilus_burn_get_drive_monitor ();
+    drives = nautilus_burn_drive_monitor_get_drives (monitor);
+    if (drives == NULL)
+      return NULL;
+
+    drive = drives->data;
+    default_device = nautilus_burn_drive_get_device (drive);
+
+    g_list_foreach (drives, (GFunc)nautilus_burn_drive_unref, NULL);
+    g_list_free (drives);
   }
   return default_device;
 }
@@ -1238,7 +1307,7 @@ int main (int argc, char **argv)
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  ctx = g_option_context_new ("- Sound Juicer");
+  ctx = g_option_context_new (_("- Extract music from your CDs"));
   g_option_context_add_main_entries (ctx, entries, GETTEXT_PACKAGE);  
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   g_option_context_set_ignore_unknown_options (ctx, TRUE);
@@ -1252,8 +1321,12 @@ int main (int argc, char **argv)
 
   g_set_application_name (_("Sound Juicer"));
 
+  sj_debug_init ();
+
   sj_stock_init ();
 
+  nautilus_burn_init ();
+
   gtk_window_set_default_icon_name ("sound-juicer");
   
   connection = bacon_message_connection_new ("sound-juicer");
@@ -1290,6 +1363,7 @@ int main (int argc, char **argv)
   gconf_client_notify_add (gconf_client, GCONF_PARANOIA, paranoia_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_PATH_PATTERN, path_pattern_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_FILE_PATTERN, file_pattern_changed_cb, NULL, NULL, NULL);
+  gconf_client_notify_add (gconf_client, GCONF_AUDIO_VOLUME, audio_volume_changed_cb, NULL, NULL, NULL);
   gconf_client_add_dir (gconf_client, GCONF_PROXY_ROOT, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
   gconf_client_notify_add (gconf_client, GCONF_HTTP_PROXY_ENABLE, http_proxy_enable_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_HTTP_PROXY, http_proxy_changed_cb, NULL, NULL, NULL);
@@ -1446,6 +1520,7 @@ int main (int argc, char **argv)
   paranoia_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PARANOIA, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
+  audio_volume_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_VOLUME, NULL, TRUE, NULL), NULL);
   if (device == NULL) {
     device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
   } else {
@@ -1461,6 +1536,8 @@ int main (int argc, char **argv)
   gtk_widget_show (main_window);
   gtk_main ();
 
+  nautilus_burn_shutdown ();
+
   g_object_unref (metadata);
   g_object_unref (extractor);
   g_object_unref (gconf_client);
